==> 1.0.5.tar.gz: lib/src/backdrop.dart <==
import 'package:backdrop_material_widget/src/utility/persistent_bottom_sheet.dart';
import 'package:flutter/material.dart';

import 'package:backdrop_material_widget/src/backdrop_page.dart';

///Backdrop is InheritedWidget for provide BackdropScaffold state in child widgets.
class Backdrop extends InheritedWidget {
  final BackdropScaffoldState state;

  Backdrop({Key key, @required this.state, @required Widget child}) : super(key: key, child: child);

  static BackdropScaffoldState of(BuildContext context) => context.dependOnInheritedWidgetOfExactType<Backdrop>().state;

  @override
  bool updateShouldNotify(Backdrop oldWidget) => state != oldWidget.state;
}

///BackdropScaffold is wrapper over Scaffold with behavior Backdrop Material Component
class BackdropScaffold extends StatefulWidget {
  ///Default constructor
  ///If [titleAppBar], [actions], [frontWidgetBuilder], [backPanelBuilder]  is empty this will throw an exception.
  ///If [actions] is empty this will not throw an exception.
  BackdropScaffold({
    Key key,
    @required this.titleAppBar,
    @required this.frontWidgetBuilder,
    @required this.backPanelBuilder,
    this.actions = const <Widget>[],
  })  : assert(frontWidgetBuilder != null),
        assert(backPanelBuilder != null),
        assert(titleAppBar != null),
        destinations = [],
        super(key: key);
  final Widget titleAppBar;
  final WidgetBuilder frontWidgetBuilder; //Front panel widget
  final WidgetBuilder backPanelBuilder; //Back panel builder

  ///Constructor navigation for navigation mode.
  ///[destinations] is required argument. If [destinations] is empty this will throw an exception.
  ///If [actions] is empty this will not throw an exception.
  BackdropScaffold.navigation({
    Key key,
    @required this.titleAppBar,
    @required this.destinations,
    this.actions,
  })  : assert(destinations != null || destinations.isEmpty),
        assert(titleAppBar != null),
        frontWidgetBuilder = null,
        backPanelBuilder = null,
        super(key: key);

  final List<NavigationDestination> destinations; //List navigation pointers
  final List<Widget> actions; //Actions

  @override
  State<BackdropScaffold> createState() => BackdropScaffoldState();
}

class BackdropScaffoldState extends State<BackdropScaffold> with SingleTickerProviderStateMixin {
  final GlobalKey<ScaffoldState> _keyScaffold = GlobalKey(); //Scaffold key for manage state
  final GlobalKey _keyBackPanel = GlobalKey(); //Key is BackPanel widget for height take
  final ValueNotifier<bool> _openNotifier = ValueNotifier<bool>(false);
  NavigationDestination current;
  double heightBackPanel = 0.0; //For calc height front panel
  int indexPage = 0; //For navigation mode

  AnimationController _generalAnimationController;

  @override
  void initState() {
    _generalAnimationController =  AnimationController(vsync: this, duration: const Duration(milliseconds: 100), value: 1.0);
    if(widget.destinations.isNotEmpty){
      current = widget.destinations[0];
    }
    super.initState();
    WidgetsBinding.instance.addPostFrameCallback((_) {
      setState(() {
        heightBackPanel = (_keyBackPanel.currentContext?.findRenderObject() as RenderBox)
            ?.size
            ?.height ?? 0.0; //Take
      });
    });
  }

  @override
  void dispose() {
    _generalAnimationController.dispose();
    super.dispose();
  }


  bool get _panelIsVisible => _generalAnimationController.status == AnimationStatus.completed || _generalAnimationController.status == AnimationStatus.forward;

  //Calculating height elements and animated
  Animation<RelativeRect> _getPanelAnimation(BoxConstraints constraints) {
    final double height = constraints.biggest.height;
    double backPanelHeight = 0;
    double frontPanelHeight = 0;

    if (heightBackPanel < height - 46) {
      backPanelHeight = heightBackPanel;
      frontPanelHeight = heightBackPanel * -1;
    } else {
      backPanelHeight = height - 46;
      frontPanelHeight = backPanelHeight * -1;
    }
    return RelativeRectTween(
        begin: RelativeRect.fromLTRB(0.0, backPanelHeight, 0.0, frontPanelHeight),
        end: const RelativeRect.fromLTRB(0.0, 0.0, 0.0, 0.0))
        .animate(CurvedAnimation(parent: _generalAnimationController, curve: Curves.linear));
  }

  //Navigation mode only used for routing on destinations
  void _route(int index) {
    if (index != indexPage) {
      setState(() {
        indexPage = index;
      });
      updateCurrentDestination(index);
      flingFrontPanel();
    }
  }

  void updateCurrentDestination(index) => setState(() {
    current = widget.destinations[index];
  });

  ///Will open front panel if her is close
  void flingFrontPanel() {
    if (!_panelIsVisible)
      _fling();
  }

  //Icon animation
  void _fling() {
    _generalAnimationController.fling(velocity: _panelIsVisible ? -1.0 : 1.0);
    _openNotifier.value = !_panelIsVisible;
  }

  //Build front and back panels
  Widget _panelsBuilder(BuildContext context, BoxConstraints constraints) {
    return Stack(
      fit: StackFit.expand,
      children: <Widget>[
        Container(
            padding: widget.destinations.isNotEmpty && widget.destinations.length >= 10
                ? const EdgeInsets.only(bottom: 46, left: 0, right: 0, top: 0)
                : EdgeInsets.zero,
            color: Theme.of(context).primaryColor,
            child: Column(
              children: <Widget>[
                Flexible(
                    key: _keyBackPanel,
                    child: widget.destinations.isEmpty
                        ? widget.backPanelBuilder(context)
                        : ListView.builder(
                      shrinkWrap: true,
                      itemCount: widget.destinations.length,
                      itemBuilder: (context, index) {
                        return GestureDetector(
                          onTap: () => _route(index),
                          child: ListTile(
                            leading: widget.destinations[index].icon,
                            title: widget.destinations[index].title,
                          ),
                        );
                      },
                    ))
              ],
            )),
        PositionedTransition(
          rect: _getPanelAnimation(constraints),
          child: Material(
            elevation: 12.0,
            borderRadius: const BorderRadius.only(
                topLeft: Radius.circular(16.0),
                topRight: Radius.circular(16.0)
            ),
            child: widget.destinations.isEmpty
                ? widget.frontWidgetBuilder(context)
                : AnimatedSwitcher(
              duration: const Duration(milliseconds: 300),
              switchInCurve: Curves.easeInCirc,
              switchOutCurve: Curves.easeOutCirc,
              child: BackdropPage(
                subTitle: widget.destinations[indexPage].subTitle,
                child: widget.destinations[indexPage].child,
                fling: flingFrontPanel,
              ),
            ),
          ),
        )
      ],
    );
  }

  ///Show persistent bottom sheet.
  ///Use [ControllerBottomSheet] for taking data or close sheet
  Future<T> showPersistentBottomSheet<T>({@required BuildContext context, @required WidgetBuilder builder, Color background, ShapeBorder shape, double elevation, Clip clipBehavior}){
    if(!_panelIsVisible)
      _fling();
    return showCustomBottomSheet<T>(
        context: context,
        builder: builder,
        backgroundColor: background,
        shape: shape,
        elevation: elevation,
        clipBehavior: clipBehavior
    );
  }

  ///Show SnackBar and returned ScaffoldFeatureController<SnackBar, SnackBarClosedReason>
  ScaffoldFeatureController<SnackBar, SnackBarClosedReason> callSnackBar(SnackBar snackBar) => _keyScaffold.currentState.showSnackBar(snackBar);

  @override
  Widget build(BuildContext context) {
    return Backdrop(
      state: this,
      child: Scaffold(
        key: _keyScaffold,
        resizeToAvoidBottomInset: false,
        appBar: widget.destinations.isNotEmpty ? AppBar(
          title: ValueListenableBuilder(
              valueListenable: _openNotifier,
              child: widget.titleAppBar,
              builder: (context, value, child){
                if(value)
                  return  child;
                return current.title;
              }
          ),
          leading: IconButton(
              icon: AnimatedIcon(
                  icon: AnimatedIcons.close_menu,
                  progress: _generalAnimationController.view), onPressed: _fling),
          actions: current.actions,
          elevation: 0.0,
        ) : AppBar(
          title: widget.titleAppBar,
          leading: IconButton(
              icon: AnimatedIcon(
                  icon: AnimatedIcons.close_menu,
                  progress: _generalAnimationController.view), onPressed: _fling),
          actions: widget.actions,
          elevation: 0.0,
        ),
        body: LayoutBuilder(
          builder: _panelsBuilder,
        ),
      ),
    );
  }
}

==> 1.0.6.tar.gz: lib/src/backdrop.dart <==
import 'package:backdrop_material_widget/src/utility/persistent_bottom_sheet.dart';
import 'package:flutter/material.dart';

import 'package:backdrop_material_widget/src/backdrop_page.dart';

///Backdrop is InheritedWidget for provide BackdropScaffold state in child widgets.
class Backdrop extends InheritedWidget {
  final BackdropScaffoldState state;

  Backdrop({Key key, @required this.state, @required Widget child}) : super(key: key, child: child);

  static BackdropScaffoldState of(BuildContext context) => context.dependOnInheritedWidgetOfExactType<Backdrop>().state;

  @override
  bool updateShouldNotify(Backdrop oldWidget) => state != oldWidget.state;
}

///BackdropScaffold is wrapper over Scaffold with behavior Backdrop Material Component
class BackdropScaffold extends StatefulWidget {
  ///Default constructor
  ///If [titleAppBar], [actions], [frontWidgetBuilder], [backPanelBuilder]  is empty this will throw an exception.
  ///If [actions] is empty this will not throw an exception.
  BackdropScaffold({
    Key key,
    @required this.titleAppBar,
    @required this.frontWidgetBuilder,
    @required this.backPanelBuilder,
    this.actions = const <Widget>[],
  })  : assert(frontWidgetBuilder != null),
        assert(backPanelBuilder != null),
        assert(titleAppBar != null),
        destinations = [],
        super(key: key);
  final Widget titleAppBar;
  final WidgetBuilder frontWidgetBuilder; //Front panel widget
  final WidgetBuilder backPanelBuilder; //Back panel builder

  ///Constructor navigation for navigation mode.
  ///[destinations] is required argument. If [destinations] is empty this will throw an exception.
  ///If [actions] is empty this will not throw an exception.
  BackdropScaffold.navigation({
    Key key,
    @required this.titleAppBar,
    @required this.destinations,
    this.actions,
  })  : assert(destinations != null || destinations.isEmpty),
        assert(titleAppBar != null),
        frontWidgetBuilder = null,
        backPanelBuilder = null,
        super(key: key);

  final List<NavigationDestination> destinations; //List navigation pointers
  final List<Widget> actions; //Actions

  @override
  State<BackdropScaffold> createState() => BackdropScaffoldState();
}

class BackdropScaffoldState extends State<BackdropScaffold> with SingleTickerProviderStateMixin {
  final GlobalKey<ScaffoldState> _keyScaffold = GlobalKey(); //Scaffold key for manage state
  final GlobalKey _keyBackPanel = GlobalKey(); //Key is BackPanel widget for height take
  final ValueNotifier<bool> _openNotifier = ValueNotifier<bool>(false);
  NavigationDestination current;
  double heightBackPanel = 0.0; //For calc height front panel
  int indexPage = 0; //For navigation mode

  AnimationController _generalAnimationController;

  @override
  void initState() {
    _generalAnimationController =  AnimationController(vsync: this, duration: const Duration(milliseconds: 100), value: 1.0);
    if(widget.destinations.isNotEmpty){
      current = widget.destinations[0];
    }
    super.initState();
    WidgetsBinding.instance.addPostFrameCallback((_) {
      setState(() {
        heightBackPanel = (_keyBackPanel.currentContext?.findRenderObject() as RenderBox)
            ?.size
            ?.height ?? 0.0; //Take
      });
    });
  }

  @override
  void dispose() {
    _generalAnimationController.dispose();
    super.dispose();
  }


  bool get _panelIsVisible => _generalAnimationController.status == AnimationStatus.completed || _generalAnimationController.status == AnimationStatus.forward;

  //Calculating height elements and animated
  Animation<RelativeRect> _getPanelAnimation(BoxConstraints constraints) {
    final double height = constraints.biggest.height;
    double backPanelHeight = 0;
    double frontPanelHeight = 0;

    if (heightBackPanel < height - 46) {
      backPanelHeight = heightBackPanel;
      frontPanelHeight = heightBackPanel * -1;
    } else {
      backPanelHeight = height - 46;
      frontPanelHeight = backPanelHeight * -1;
    }
    return RelativeRectTween(
        begin: RelativeRect.fromLTRB(0.0, backPanelHeight, 0.0, frontPanelHeight),
        end: const RelativeRect.fromLTRB(0.0, 0.0, 0.0, 0.0))
        .animate(CurvedAnimation(parent: _generalAnimationController, curve: Curves.linear));
  }

  //Navigation mode only used for routing on destinations
  void _route(int index) {
    if (index != indexPage) {
      setState(() {
        indexPage = index;
      });
      updateCurrentDestination(index);
      flingFrontPanel();
    }
  }

  void updateCurrentDestination(index) => setState(() {
    current = widget.destinations[index];
  });

  ///Will open front panel if her is close
  void flingFrontPanel() {
    if (!_panelIsVisible)
      _fling();
  }

  //Icon animation
  void _fling() {
    _generalAnimationController.fling(velocity: _panelIsVisible ? -1.0 : 1.0);
    _openNotifier.value = !_panelIsVisible;
  }

  //Build front and back panels
  Widget _panelsBuilder(BuildContext context, BoxConstraints constraints) {
    return Stack(
      fit: StackFit.expand,
      children: <Widget>[
        Container(
            padding: widget.destinations.isNotEmpty && widget.destinations.length >= 10
                ? const EdgeInsets.only(bottom: 46, left: 0, right: 0, top: 0)
                : EdgeInsets.zero,
            color: Theme.of(context).primaryColor,
            child: Column(
              children: <Widget>[
                Flexible(
                    key: _keyBackPanel,
                    child: widget.destinations.isEmpty
                        ? widget.backPanelBuilder(context)
                        : ListTileTheme(
                          textColor: Theme.of(context).primaryTextTheme.bodyText2.color,
                          iconColor: Theme.of(context).primaryIconTheme.color,
                          child: ListView.builder(
                            shrinkWrap: true,
                            itemCount: widget.destinations.length,
                            itemBuilder: (context, index) {
                              return ListTile(
                                leading: widget.destinations[index].icon,
                                title: widget.destinations[index].title,
                                onTap: () => _route(index),
                              );
                            },
                          ),
                    )
                )
              ],
            )),
        PositionedTransition(
          rect: _getPanelAnimation(constraints),
          child: Material(
            elevation: 12.0,
            borderRadius: const BorderRadius.only(
                topLeft: Radius.circular(16.0),
                topRight: Radius.circular(16.0)
            ),
            child: widget.destinations.isEmpty
                ? widget.frontWidgetBuilder(context)
                : AnimatedSwitcher(
              duration: const Duration(milliseconds: 300),
              switchInCurve: Curves.easeInCirc,
              switchOutCurve: Curves.easeOutCirc,
              child: BackdropPage(
                subTitle: widget.destinations[indexPage].subTitle,
                child: widget.destinations[indexPage].child,
                fling: flingFrontPanel,
              ),
            ),
          ),
        )
      ],
    );
  }

  ///Show persistent bottom sheet.
  ///Use [ControllerBottomSheet] for taking data or close sheet
  Future<T> showPersistentBottomSheet<T>({@required BuildContext context, @required WidgetBuilder builder, Color background, ShapeBorder shape, double elevation, Clip clipBehavior}){
    if(!_panelIsVisible)
      _fling();
    return showCustomBottomSheet<T>(
        context: context,
        builder: builder,
        backgroundColor: background,
        shape: shape,
        elevation: elevation,
        clipBehavior: clipBehavior
    );
  }

  ///Show SnackBar and returned ScaffoldFeatureController<SnackBar, SnackBarClosedReason>
  ScaffoldFeatureController<SnackBar, SnackBarClosedReason> callSnackBar(SnackBar snackBar) => _keyScaffold.currentState.showSnackBar(snackBar);

  @override
  Widget build(BuildContext context) {
    return Backdrop(
      state: this,
      child: Scaffold(
        key: _keyScaffold,
        resizeToAvoidBottomInset: false,
        appBar: widget.destinations.isNotEmpty ? AppBar(
          title: ValueListenableBuilder(
              valueListenable: _openNotifier,
              child: widget.titleAppBar,
              builder: (context, value, child) => value ? child : current.title
          ),
          leading: IconButton(icon: AnimatedIcon(icon: AnimatedIcons.close_menu, progress: _generalAnimationController.view), onPressed: _fling),
          actions: current.actions,
          elevation: 0.0,
        ) : AppBar(
          title: widget.titleAppBar,
          leading: IconButton(icon: AnimatedIcon(icon: AnimatedIcons.close_menu, progress: _generalAnimationController.view), onPressed: _fling),
          actions: widget.actions,
          elevation: 0.0,
        ),
        body: LayoutBuilder(
          builder: _panelsBuilder,
        ),
      ),
    );
  }
}
